extends Node3D

enum State { idle, moveRoll, monster, flee }
enum Tile { stone, pumpkin, candy, monster1, monster2, monster3 }

var mats := ["pebbles", "pumpkin", "candy", "monster1", "monster2", "monster3"]

@onready var player: MeshInstance3D = $"../Player"
@onready var info_text: Label = $"../InfoText"
@onready var score_text: Label = $"../ScoreText"
@onready var button_1: Button = $"../Button1"
@onready var button_2: Button = $"../Button2"
@onready var button_3: Button = $"../Button3"
@onready var die_obj: Node3D = $"../DieObj"

var tiles := []
var tile_type := []
var tiles_nr = 20
var state = State.idle

var die_value := 0
var die_rotations = [[0, 0, 0 ], [-90, 0, 0 ], [0, 0, 90 ], [0, 0, -90 ], [ 90, 0, 0 ], [180, 0, 0 ]]

var player_pos := 0
var player_offset = Vector3(0, 0, 0)

# Called when the node enters the scene tree for the first time.
func _ready() -> void:
	tiles.resize(tiles_nr)
	tile_type.resize(tiles_nr)
	for i in len(tiles):
		# use the name of the tiles to populate the array
		tiles[i] = get_node("../Path/tile" + str(i))
		if tiles[i]:
			# use the material's name to set the type
			var mat = tiles[i].material_override.albedo_texture.load_path
			tile_type[i] = Tile.stone
			for j in len(mats):
				if mat.contains(mats[j]):
					tile_type[i] = j as Tile
	roll_die()
	player_offset = tiles[0].position - player.position
	place_player(0)     # starting position for the game
	state = State.idle # initial state
	set_buttons()
	info_text.text = ""

# Called every frame. 'delta' is the elapsed time since the previous frame.
func _process(delta: float) -> void:
	pass

# Sets the rotation of the die object based on the value of the die.
func set_die(value):
	if 1 <= value and value <= 6:
		var x = die_rotations[value - 1][0]
		var y = die_rotations[value - 1][1]
		var z = die_rotations[value - 1][2]
		die_obj.rotation = Vector3(x, y, z)

func roll_die():
	die_value = randi_range(1, 6)
	set_die(die_value)
	info_text.text = "Roll: " + str(die_value)
	
func place_player(k):
	player_pos = k
	player.position = tiles[k].position - player_offset

func set_buttons():
	button_1.text = "";
	button_2.text = "";
	button_3.text = "";

	if state == State.idle:
		button_1.text = "Roll"
	
# first button action
func action1():
	if state == State.idle:
		roll_die()
		state = State.moveRoll     # next state
	set_buttons()
